"use strict";
/**
 * Copyright 2023 actions-toolkit authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Git = void 0;
const core = __importStar(require("@actions/core"));
const core_1 = require("@octokit/core");
const plugin_rest_endpoint_methods_1 = require("@octokit/plugin-rest-endpoint-methods");
const exec_1 = require("./exec");
const github_1 = require("./github");
const context_1 = require("@actions/github/lib/context");
class Git {
    static async context() {
        const ctx = new context_1.Context();
        ctx.ref = await Git.ref();
        ctx.sha = await Git.fullCommit();
        return ctx;
    }
    static async isInsideWorkTree() {
        return await Git.exec(['rev-parse', '--is-inside-work-tree'])
            .then(out => {
            return out === 'true';
        })
            .catch(() => {
            return false;
        });
    }
    static async remoteSha(repo, ref, token) {
        const repoMatch = repo.match(/github.com\/([^/]+)\/([^/]+?)(?:\.git)?(\/|$)/);
        // if we have a token and this is a GitHub repo we can use the GitHub API
        if (token && repoMatch) {
            core.setSecret(token);
            const octokit = new (core_1.Octokit.plugin(plugin_rest_endpoint_methods_1.restEndpointMethods).defaults({
                baseUrl: github_1.GitHub.apiURL
            }))({ auth: token });
            const [owner, repoName] = repoMatch.slice(1, 3);
            try {
                return (await octokit.rest.repos.listCommits({
                    owner: owner,
                    repo: repoName,
                    sha: ref,
                    per_page: 1
                })).data[0].sha;
            }
            catch (e) {
                throw new Error(`Cannot find remote ref for ${repo}#${ref}: ${e.message}`);
            }
        }
        // otherwise we fall back to git ls-remote
        return await Git.exec(['ls-remote', repo, ref]).then(out => {
            const [rsha] = out.split(/[\s\t]/);
            if (rsha.length == 0) {
                throw new Error(`Cannot find remote ref for ${repo}#${ref}`);
            }
            return rsha;
        });
    }
    static async remoteURL() {
        return await Git.exec(['remote', 'get-url', 'origin']).then(rurl => {
            if (rurl.length == 0) {
                return Git.exec(['remote', 'get-url', 'upstream']).then(rurl => {
                    if (rurl.length == 0) {
                        throw new Error(`Cannot find remote URL for origin or upstream`);
                    }
                    return rurl;
                });
            }
            return rurl;
        });
    }
    static async ref() {
        const isHeadDetached = await Git.isHeadDetached();
        if (isHeadDetached) {
            return await Git.getDetachedRef();
        }
        return await Git.exec(['symbolic-ref', 'HEAD']);
    }
    static async fullCommit() {
        return await Git.exec(['show', '--format=%H', 'HEAD', '--quiet', '--']);
    }
    static async shortCommit() {
        return await Git.exec(['show', '--format=%h', 'HEAD', '--quiet', '--']);
    }
    static async tag() {
        return await Git.exec(['tag', '--points-at', 'HEAD', '--sort', '-version:creatordate']).then(tags => {
            if (tags.length == 0) {
                return Git.exec(['describe', '--tags', '--abbrev=0']);
            }
            return tags.split('\n')[0];
        });
    }
    static async isHeadDetached() {
        return await Git.exec(['branch', '--show-current']).then(res => {
            return res.length == 0;
        });
    }
    static async getDetachedRef() {
        const res = await Git.exec(['show', '-s', '--pretty=%D']);
        // Can be "HEAD, <tagname>" or "grafted, HEAD, <tagname>"
        const refMatch = res.match(/^(grafted, )?HEAD, (.*)$/);
        if (!refMatch || !refMatch[2]) {
            throw new Error(`Cannot find detached HEAD ref in "${res}"`);
        }
        const ref = refMatch[2].trim();
        // Tag refs are formatted as "tag: <tagname>"
        if (ref.startsWith('tag: ')) {
            return `refs/tags/${ref.split(':')[1].trim()}`;
        }
        // Otherwise, it's a branch "<origin>/<branch-name>, <branch-name>"
        const branchMatch = ref.match(/^[^/]+\/[^/]+, (.+)$/);
        if (branchMatch) {
            return `refs/heads/${branchMatch[1].trim()}`;
        }
        throw new Error(`Unsupported detached HEAD ref in "${res}"`);
    }
    static async exec(args = []) {
        return await exec_1.Exec.getExecOutput(`git`, args, {
            ignoreReturnCode: true,
            silent: true
        }).then(res => {
            if (res.stderr.length > 0 && res.exitCode != 0) {
                throw new Error(res.stderr);
            }
            return res.stdout.trim();
        });
    }
}
exports.Git = Git;
//# sourceMappingURL=git.js.map